@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2013 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -41,6 +41,7 @@
 
 #define PROTO_LEN	2
 #define BIG_PROTO_LEN	3
+#define LEVENSHTEIN_LEN	3
 
 static int get_dictionary_lang(struct _xneur_handle *handle, char **word)
 {
@@ -48,7 +49,8 @@ static int get_dictionary_lang(struct _xneur_handle *handle, char **word)
 	{
 		if (handle->languages[lang].disable_auto_detection || handle->languages[lang].excluded)
 			continue;
-
+		if (strlen(word[lang]) == 0)
+			continue;
 		if (handle->languages[lang].dictionary->exist(handle->languages[lang].dictionary, word[lang], BY_REGEXP))
 		{
 			log_message(DEBUG, _("   [+] Found this word in %s language dictionary"), handle->languages[lang].name);
@@ -66,13 +68,16 @@ static int get_aspell_hits(struct _xneur_handle *handle, char **word, int **sym_
 	// check for current language first
 	if (handle->has_spell_checker[cur_lang])
 	{
-		if (strlen(word[cur_lang]) / sym_len[cur_lang][0] > 1)
+		if (strlen(word[cur_lang]) > 0)
 		{
-			if (!handle->languages[cur_lang].disable_auto_detection && !handle->languages[cur_lang].excluded &&
-			    aspell_speller_check(handle->spell_checkers[cur_lang], word[cur_lang], strlen(word[cur_lang])))
+			if (strlen(word[cur_lang]) / sym_len[cur_lang][0] > 1)
 			{
-				log_message(DEBUG, _("   [+] Found this word in %s aspell dictionary"), handle->languages[cur_lang].name);
-				return cur_lang;
+				if (!handle->languages[cur_lang].disable_auto_detection && !handle->languages[cur_lang].excluded &&
+					aspell_speller_check(handle->spell_checkers[cur_lang], word[cur_lang], strlen(word[cur_lang])))
+				{
+					log_message(DEBUG, _("   [+] Found this word in %s aspell dictionary"), handle->languages[cur_lang].name);
+					return cur_lang;
+				}
 			}
 		}
 	}
@@ -81,13 +86,15 @@ static int get_aspell_hits(struct _xneur_handle *handle, char **word, int **sym_
 		log_message(DEBUG, _("   [!] Now we don't support aspell dictionary for %s layout"), handle->languages[cur_lang].name);
 	}
 
-
 	// check for another languages
 	for (int lang = 0; lang < handle->total_languages; lang++)
 	{
 		if (handle->languages[lang].disable_auto_detection || handle->languages[lang].excluded || lang == cur_lang)
 			continue;
 
+		if (strlen(word[lang]) == 0)
+			continue;
+
 		if (!handle->has_spell_checker[lang])
 		{
 			log_message(DEBUG, _("   [!] Now we don't support aspell dictionary for %s layout"), handle->languages[lang].name);
@@ -116,14 +123,16 @@ static int get_enchant_hits(struct _xneur_handle *handle, char **word, int **sym
 	// check for current language first
 	if (handle->has_enchant_checker[cur_lang])
 	{
-		
-		if (strlen(word[cur_lang]) / sym_len[cur_lang][0] > 1)
+		if (strlen(word[cur_lang]) > 0)
 		{
-			if (!handle->languages[cur_lang].disable_auto_detection && !handle->languages[cur_lang].excluded &&
-			    !enchant_dict_check(handle->enchant_dicts[cur_lang], word[cur_lang], strlen(word[cur_lang])))
+			if ((int)(strlen(word[cur_lang]) / sym_len[cur_lang][0]) > 1)
 			{
-				log_message(DEBUG, _("   [+] Found this word in %s enchant wrapper dictionary"), handle->languages[cur_lang].name);
-				return cur_lang;
+				if (!handle->languages[cur_lang].disable_auto_detection && !handle->languages[cur_lang].excluded &&
+					!enchant_dict_check(handle->enchant_dicts[cur_lang], word[cur_lang], strlen(word[cur_lang])))
+				{
+					log_message(DEBUG, _("   [+] Found this word in %s enchant wrapper dictionary"), handle->languages[cur_lang].name);
+					return cur_lang;
+				}
 			}
 		}
 	}
@@ -132,13 +141,15 @@ static int get_enchant_hits(struct _xneur_handle *handle, char **word, int **sym
 		log_message(DEBUG, _("   [!] Now we don't support enchant wrapper dictionary for %s layout"), handle->languages[cur_lang].name);
 	}
 
-
 	// check for another languages
 	for (int lang = 0; lang < handle->total_languages; lang++)
 	{
 		if (handle->languages[lang].disable_auto_detection || handle->languages[lang].excluded || lang == cur_lang || (strlen(word[lang]) <= 0)) 
 			continue;
 
+		if (strlen(word[lang]) == 0)
+			continue;
+
 		if (!handle->has_enchant_checker[lang])
 		{
 			log_message(DEBUG, _("   [!] Now we don't support enchant wrapper dictionary for %s layout"), handle->languages[lang].name);
@@ -243,6 +254,9 @@ static int get_proto_lang(struct _xneur_handle *handle, char **word, int **sym_l
 		if ((lang == cur_lang) || (handle->languages[lang].disable_auto_detection) || (handle->languages[lang].excluded))
 			continue;
 
+		if (strlen(word[lang]) == 0)
+			continue;
+
 		int hits = get_proto_hits_function(handle, word[lang], sym_len[lang], len, offset, lang);
 		if (hits != 0)
 		{
@@ -260,33 +274,34 @@ static int get_proto_lang(struct _xneur_handle *handle, char **word, int **sym_l
 
 static int get_similar_words(struct _xneur_handle *handle, struct _buffer *p)
 {
-	int min_levenshtein = 3;
+	int min_levenshtein = LEVENSHTEIN_LEN;
 	char *possible_words = NULL;
 	int possible_lang = NO_LANGUAGE;
 	
 	int lang = 0;
 	for (lang = 0; lang < handle->total_languages; lang++)
 	{
-		char *word = strdup(get_last_word(p->i18n_content[lang].content));
+		char *word = strdup(p->get_last_word(p, p->i18n_content[lang].content));
+		if (word == NULL)
+			continue;
+		
 		del_final_numeric_char(word);
 		
 		if (handle->languages[lang].disable_auto_detection || handle->languages[lang].excluded)
 		{
 			if (possible_words != NULL)
 				free (possible_words);
-			if (word != NULL)
-				free(word);
+			free(word);
 			continue;
 		}
 
-		int word_len = strlen(get_last_word(p->content));
+		int word_len = strlen(p->get_last_word(p, p->content));
 		
 		if ((word_len > 250) || (word_len < 2))
 		{
 			if (possible_words != NULL)
 				free (possible_words);
-			if (word != NULL)
-				free(word);
+			free(word);
 			continue;
 		}
 
@@ -356,7 +371,6 @@ static int get_similar_words(struct _xneur_handle *handle, struct _buffer *p)
 		int i = 0;
 		while ((sugg_word = aspell_string_enumeration_next (elements)) != NULL)
 		{
-			
 			int tmp_levenshtein = levenshtein(word+offset, sugg_word);
 			if (tmp_levenshtein < min_levenshtein)
 			{
@@ -377,16 +391,16 @@ static int get_similar_words(struct _xneur_handle *handle, struct _buffer *p)
 			free(word);
 	}
 	
-	if (possible_words != NULL)
-	{
-		log_message(DEBUG, _("   [+] Found suggest word '%s' in %s dictionary (Levenshtein distance = %d)"),  
-		            possible_words, handle->languages[possible_lang].name, min_levenshtein);
-		free (possible_words); 
-	}
-	else
+	if (possible_words == NULL)
 	{
 		log_message(DEBUG, _("   [-] This word has no suggest for all dictionaries")); 
+		return possible_lang;
+
 	}
+	
+	log_message(DEBUG, _("   [+] Found suggest word '%s' in %s dictionary (Levenshtein distance = %d)"),  
+						possible_words, handle->languages[possible_lang].name, min_levenshtein);
+	free (possible_words); 
 	return possible_lang;
 }
 
@@ -401,7 +415,7 @@ int check_lang(struct _xneur_handle *handle, struct _buffer *p, int cur_lang)
 	log_message(DEBUG, _("Processing word:"));
 	for (int i = 0; i < handle->total_languages; i++)
 	{
-		word[i] = strdup(get_last_word(p->i18n_content[i].content));
+		word[i] = strdup(p->get_last_word(p, p->i18n_content[i].content));
 		word_base[i] = word[i];
 		del_final_numeric_char(word[i]);
 
@@ -413,21 +427,21 @@ int check_lang(struct _xneur_handle *handle, struct _buffer *p, int cur_lang)
 		}
 		word[i] = word[i] + offset;
 		
-		word_unchanged[i] = strdup(get_last_word(p->i18n_content[i].content_unchanged));
+		word_unchanged[i] = strdup(p->get_last_word(p, p->i18n_content[i].content_unchanged));
 		word_unchanged_base[i] = word_unchanged[i];
 		word_unchanged[i] = word_unchanged[i] + offset;
 		del_final_numeric_char(word_unchanged[i]);
 		
 		log_message(DEBUG, _("   '%s' on layout '%s'"), word_unchanged[i], handle->languages[i].dir);
 
-		sym_len[i] = p->i18n_content[i].symbol_len + get_last_word_offset(p->content, strlen(p->content));
+		sym_len[i] = p->i18n_content[i].symbol_len + p->get_last_word_offset(p, p->content, strlen(p->content));
 	}
 
 	log_message(DEBUG, _("Start word processing..."));
 
 	// Check by dictionary
 	int lang = get_dictionary_lang(handle, word);
-	
+
 #ifdef WITH_ENCHANT
 	// Check by enchant wrapper
 	if (lang == NO_LANGUAGE)
@@ -442,7 +456,7 @@ int check_lang(struct _xneur_handle *handle, struct _buffer *p, int cur_lang)
 	
 	// If not found in dictionary, try to find in proto
 	int len = strlen(p->content);
-	int offset = get_last_word_offset(p->content, len);
+	int offset = p->get_last_word_offset(p, p->content, len);
 	if (lang == NO_LANGUAGE)
 		lang = get_proto_lang(handle, word, sym_len, len, offset, cur_lang, PROTO_LEN);
 
@@ -476,7 +490,7 @@ int check_lang_with_similar_words (struct _xneur_handle *handle, struct _buffer
 	log_message(DEBUG, _("Processing word:"));
 	for (int i = 0; i < handle->total_languages; i++)
 	{
-		word[i] = strdup(get_last_word(p->i18n_content[i].content));
+		word[i] = strdup(p->get_last_word(p, p->i18n_content[i].content));
 		word_base[i] = word[i];
 		del_final_numeric_char(word[i]);
 
@@ -488,14 +502,14 @@ int check_lang_with_similar_words (struct _xneur_handle *handle, struct _buffer
 		}
 		word[i] = word[i] + offset;
 		
-		word_unchanged[i] = strdup(get_last_word(p->i18n_content[i].content_unchanged));
+		word_unchanged[i] = strdup(p->get_last_word(p, p->i18n_content[i].content_unchanged));
 		word_unchanged_base[i] = word_unchanged[i];
 		word_unchanged[i] = word_unchanged[i] + offset;
 		del_final_numeric_char(word_unchanged[i]);
 		
 		log_message(DEBUG, _("   '%s' on layout '%s'"), word_unchanged[i], handle->languages[i].dir);
 
-		sym_len[i] = p->i18n_content[i].symbol_len + get_last_word_offset(p->content, strlen(p->content));
+		sym_len[i] = p->i18n_content[i].symbol_len + p->get_last_word_offset(p, p->content, strlen(p->content));
 	}
 
 	log_message(DEBUG, _("Start word processing..."));
@@ -521,7 +535,7 @@ int check_lang_with_similar_words (struct _xneur_handle *handle, struct _buffer
 	
 	// If not found in dictionary, try to find in proto
 	int len = strlen(p->content);
-	int offset = get_last_word_offset(p->content, len);
+	int offset = p->get_last_word_offset(p, p->content, len);
 	if (lang == NO_LANGUAGE)
 		lang = get_proto_lang(handle, word, sym_len, len, offset, cur_lang, PROTO_LEN);
 
