@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2013 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -21,6 +21,8 @@
 #  include "config.h"
 #endif
 
+#define XK_PUBLISHING
+#include <X11/keysym.h>
 #include <X11/XKBlib.h>
 
 #include <stdlib.h>
@@ -39,7 +41,7 @@
 
 #include "xnconfig.h"
 
-#define LIBRARY_VERSION_MAJOR		19
+#define LIBRARY_VERSION_MAJOR		20
 #define LIBRARY_VERSION_MINOR		0
 #define OPTIONS_DELIMETER		" "
 
@@ -59,27 +61,27 @@ static const char *option_names[] = 	{
 						"LogSave", "ReplaceAbbreviation",
 						"ReplaceAbbreviationIgnoreLayout", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter",
 						"FlushBufferWhenPressEnter", "TroubleshootEnter", "AddAction",
-						"ShowOSD", "AddOSD", "FontOSD", "ShowPopup", "AddPopup", 
+						"ShowOSD", "AddOSD", "FontOSD", "ShowPopup", "AddPopup",
 	                    "CorrectSpaceWithPunctuation", "AddSpaceAfterAutocompletion", "LoadModule",
 						"LogSize", "LogMail", "LogHostIP", "SoundVolumePercent",
 						"TroubleshootBackspace", "TroubleshootLeftArrow", "TroubleshootRightArrow",
-						"TroubleshootUpArrow", "TroubleshootDownArrow", "TroubleshootDelete", 
+						"TroubleshootUpArrow", "TroubleshootDownArrow", "TroubleshootDelete",
 						"TroubleshootSwitch", "TroubleshootFullScreen",
 						"SetDontSendKeyReleaseApp", "LogPort", "RotateLayoutAfterChangeSelectedMode", "CorrectCapitalLetterAfterDot",
 						"FlushBufferWhenPressEscape", "CompatibilityWithCompletion", "TrackingInput", "TrackingMouse",
 						"PopupExpireTimeout", "CorrectTwoSpaceWithCommaAndSpace","CorrectTwoMinusWithDash",
 						"CorrectCWithCopyright", "CorrectTMWithTrademark", "CorrectRWithRegistered",
 						"CorrectDashWithEmDash","CorrectThreePointsWithEllipsis", "CorrectMisprint", "CheckSimilarWords",
-						"TroubleshootTab", "DelaySendingEventApp" 
+						"TroubleshootTab", "DelaySendingEventApp", "Delimeter"
 					};
 static const char *action_names[] =	{
 						"ChangeWord", "TranslitWord", "ChangecaseWord", "PreviewChangeWord",
-						"ChangeString", 
+						"ChangeString",
 						"ChangeSelected", "TranslitSelected", "ChangecaseSelected", "PreviewChangeSelected",
 						"ChangeClipboard", "TranslitClipboard", "ChangecaseClipboard", "PreviewChangeClipboard",
 						"EnableLayout1", "EnableLayout2", "EnableLayout3", "EnableLayout4",
 						"RotateLayout","RotateLayoutBack","ReplaceAbbreviation", "AutocompletionConfirmation",
-						"RotateAutocompletion", "BlockKeyboardAndMouseEvents", "InsertDate"
+						"RotateAutocompletion", "InsertDate"
 					};
 static const char *notify_names[] =	{
 						"XneurStart", "XneurReload", "XneurStop",
@@ -89,11 +91,11 @@ static const char *notify_names[] =	{
 						"ChangeString",
 						"ChangeSelected", "TranslitSelected", "ChangecaseSelected", "PreviewChangeSelected",
 						"ChangeClipboard", "TranslitClipboard", "ChangecaseClipboard",  "PreviewChangeClipboard",
-						"ReplaceAbbreviation", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter", 
-						"CorrectTwoSpaceWithCommaAndSpace", "CorrectTwoMinusWithDash", "CorrectCWithCopyright", 
+						"ReplaceAbbreviation", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter",
+						"CorrectTwoSpaceWithCommaAndSpace", "CorrectTwoMinusWithDash", "CorrectCWithCopyright",
 						"CorrectTMWithTrademark", "CorrectRWithRegistered", "CorrectThreePointsWithEllipsis",
 						"CorrectMisprint",
-						"ExecuteUserAction","BlockKeyboardAndMouseEvents", "UnblockKeyboardAndMouseEvents"
+						"ExecuteUserAction", "UnblockKeyboardAndMouseEvents"
 					};
 
 pid_t getsid(pid_t pid);
@@ -138,6 +140,80 @@ static int xneur_config_get_status(char *str)
 	return -1;
 }
 
+static struct _xneur_user_action * one_more_user_action(struct _xneur_config *p)
+{
+	void *tmp = realloc(p->user_actions, (p->user_actions_count + 1) * sizeof(struct _xneur_user_action));
+	if (tmp == NULL)
+		return NULL;
+	p->user_actions = (struct _xneur_user_action *)tmp;
+	bzero(&p->user_actions[p->user_actions_count], sizeof(struct _xneur_user_action));
+	p->user_actions_count++;
+	return &(p->user_actions[p->user_actions_count - 1]);
+}
+
+static struct _xneur_action * one_more_action(struct _xneur_config *p)
+{
+	void *tmp = realloc(p->actions, (p->actions_count + 1) * sizeof(struct _xneur_action));
+	if (tmp == NULL)
+		return NULL;
+	p->actions = (struct _xneur_action *)tmp;
+	bzero(&p->actions[p->actions_count], sizeof(struct _xneur_action));
+	p->actions_count++;
+	return &(p->actions[p->actions_count - 1]);
+}
+
+static void parse_hotkey(char **line, struct _xneur_hotkey * hotkey)
+{
+	//log_message(DEBUG, _("Parsing hotkey from: '%s'"),*line);
+
+	hotkey->key = NULL;
+	while (TRUE)
+	{
+		char *oldline = NULL;
+		if (*line)
+		{
+			oldline = strdup(*line);
+		}
+		char *modifier = get_word(line);
+		if (modifier == NULL)
+			break;
+
+		if (modifier[0] == '\0')
+		{
+			free(oldline);
+			continue;
+		}
+
+		int index = get_option_index(modifier_names, modifier);
+
+		// It seems that original function could parse "Alt t" as well as "t Alt"
+		// Now I need to unify user and standard actions
+		// So, by the name of backward compatibility (hotkey->key == NULL) check is added below:
+		if (index != -1)
+		{
+			// The word is really modifier
+			hotkey->modifiers |= (1 << index);
+			//log_message(DEBUG, _("Adding modifier: '%s'"),modifier);
+			free(oldline);
+		}
+		else if (hotkey->key == NULL)
+		{
+			// The word is not modifier, it is a key and it is first non-modifier word
+			hotkey->key = strdup(modifier);
+			//log_message(DEBUG, _("Key set to: '%s'"),modifier);
+			free(oldline);
+		}
+		else
+		{
+			// The word is not modified and key is already been readed
+			*line = oldline;
+			//if (oldline)
+			//	log_message(DEBUG, _("Restoring old line: '%s'"),oldline);
+			return;
+		}
+	}
+}
+
 static void parse_line(struct _xneur_config *p, char *line)
 {
 	if (line[0] == '#')
@@ -157,13 +233,14 @@ static void parse_line(struct _xneur_config *p, char *line)
 		log_message(WARNING, _("Param mismatch for option %s"), option);
 		return;
 	}
-	
+
 	char *full_string = strdup(line);
 	char *param = get_word(&line);
 
 	if (param == NULL)
 	{
-		free(full_string);
+		if (full_string != NULL)
+			free(full_string);
 		log_message(WARNING, _("Param mismatch for option %s"), option);
 		return;
 	}
@@ -196,22 +273,28 @@ static void parse_line(struct _xneur_config *p, char *line)
 				break;
 			}
 
-			p->hotkeys[action].key = NULL;
-			while (TRUE)
+			struct _xneur_action * new_action = one_more_action(p);
+			if (new_action != NULL)
 			{
-				char *modifier = get_word(&line);
-				if (modifier == NULL)
-					break;
-
-				if (modifier[0] == '\0')
-					continue;
-
-				int index = get_option_index(modifier_names, modifier);
-				if (index == -1)
-					p->hotkeys[action].key = strdup(modifier);
-				else
-					p->hotkeys[action].modifiers |= (1 << index);
+				parse_hotkey(&line, &(new_action->hotkey));
+				new_action->action = action;
 			}
+			/*
+			if (p->hotkeys[action].key == NULL)
+			{
+			        parse_hotkey(&line, &(p->hotkeys[action]));
+			}
+			else
+			{
+				log_message(WARNING, _("More than one hotkey specified for action '%s'"),param);
+				struct _xneur_action * new_action = one_more_user_action(p);
+				if (new_action != NULL)
+				{
+					parse_hotkey(&line, &(new_action->hotkey));
+					new_action->standard_action = action;
+					new_action->name = strdup(param);
+				}
+			}*/
 
 			break;
 		}
@@ -230,12 +313,6 @@ static void parse_line(struct _xneur_config *p, char *line)
 		}
 		case 4: // Add Language
 		{
-			if (param == NULL)
-			{
-				log_message(ERROR, _("Argument number mismatch for ExcludeLanguage option"));
-				break;
-			}
-
 			for (int lang = 0; lang < p->handle->total_languages; lang++)
 			{
 				if (strcmp(p->handle->languages[lang].dir, param) == 0)
@@ -352,7 +429,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 
 			if (line == NULL)
 				break;
-			
+
 			char *param1 = get_word(&line);
 			char *param2 = get_word(&line);
 
@@ -415,8 +492,11 @@ static void parse_line(struct _xneur_config *p, char *line)
 		case 21: // Get Words for Replacing
 		{
 			char *replaced_string = escaped_sym_to_real_sym(full_string);
-			p->abbreviations->add(p->abbreviations, replaced_string);
-			free(replaced_string);
+			if (replaced_string != NULL)
+			{
+				p->abbreviations->add(p->abbreviations, replaced_string);
+				free(replaced_string);
+			}
 			break;
 		}
 		case 22: // Ignore keyboard layout for abbreviations
@@ -481,58 +561,41 @@ static void parse_line(struct _xneur_config *p, char *line)
 		}
 		case 27: // User actions
 		{
-			p->actions = (struct _xneur_action *) realloc(p->actions, (p->actions_count + 1) * sizeof(struct _xneur_action));
-			bzero(&p->actions[p->actions_count], sizeof(struct _xneur_action));
+			struct _xneur_user_action * new_user_action = one_more_user_action(p);
+			if (new_user_action == NULL)
+				break;
 
-			while (TRUE)
+			char *whole_string = full_string;
+			parse_hotkey(&whole_string,&(new_user_action->hotkey));
+			line = whole_string;
+
+			if (line != NULL)
 			{
-				if (param == NULL)
-					break;
-
-				if (param[0] == '\0')
-					continue;
-
-				int index = get_option_index(modifier_names, param);
-				if (index == -1)
+				char *cmd = strstr(line, USR_CMD_START);
+				if (cmd == NULL)
 				{
-					if (param != NULL)
-						p->actions[p->actions_count].hotkey.key = strdup(param);
-					if (line != NULL)
-					{
-						char *cmd = strstr(line, USR_CMD_START);
-						if (cmd == NULL)
-						{
-							p->actions[p->actions_count].name = NULL;
-							p->actions[p->actions_count].command = strdup(line);
-							break;
-						}
-						int len = strlen(line) - strlen(cmd);
-						p->actions[p->actions_count].name = strdup(line);
-						p->actions[p->actions_count].name[len - 1] = NULLSYM;
-
-						p->actions[p->actions_count].command = strdup(cmd + strlen(USR_CMD_START)*sizeof(char));
-						cmd = strstr(cmd + strlen(USR_CMD_START)*sizeof(char), USR_CMD_END);
-						if (cmd == NULL)
-						{
-							free(p->actions[p->actions_count].command);
-							p->actions[p->actions_count].command = NULL;
-							break;
-						}
-						len = strlen(p->actions[p->actions_count].command) - strlen(cmd);
-						p->actions[p->actions_count].command[len] = NULLSYM;
-
-						//log_message (ERROR, "\"%s\" \"%s\"", p->actions[p->actions_count].name, p->actions[p->actions_count].command);
-						
-					}
+					new_user_action->name = NULL;
+					new_user_action->command = strdup(line);
 					break;
 				}
+				int len = strlen(line) - strlen(cmd);
+				new_user_action->name = strdup(line);
+				new_user_action->name[len - 1] = NULLSYM;
 
-				p->actions[p->actions_count].hotkey.modifiers |= (1 << index);
-
-				param = get_word(&line);
+				new_user_action->command = strdup(cmd + strlen(USR_CMD_START)*sizeof(char));
+				cmd = strstr(cmd + strlen(USR_CMD_START)*sizeof(char), USR_CMD_END);
+				if (cmd == NULL)
+				{
+					if (new_user_action->command != NULL)
+						free(new_user_action->command);
+					new_user_action->command = NULL;
+					break;
+				}
+				len = strlen(new_user_action->command) - strlen(cmd);
+				new_user_action->command[len] = NULLSYM;
+				free(line);
 			}
 
-			p->actions_count++;
 			break;
 		}
 		case 28: // Show OSD
@@ -562,8 +625,10 @@ static void parse_line(struct _xneur_config *p, char *line)
 				p->osds[osd].enabled = FALSE;
 				break;
 			}
-			
+
 			char *tmp = strdup(line);
+			if (tmp == NULL)
+				break;
 			char *param1 = get_word(&line);
 
 			if (strlen(param1) != 0)
@@ -581,8 +646,9 @@ static void parse_line(struct _xneur_config *p, char *line)
 			}
 			if (p->osds[osd].file == NULL)
 				p->osds[osd].enabled = FALSE;
-			
-			free(tmp);
+
+			if (tmp != NULL)
+				free(tmp);
 
 			break;
 		}
@@ -618,8 +684,10 @@ static void parse_line(struct _xneur_config *p, char *line)
 				p->popups[popup].enabled = FALSE;
 				break;
 			}
-			
+
 			char *tmp = strdup(line);
+			if (tmp == NULL)
+				break;
 			char *param1 = get_word(&line);
 
 			if (strlen(param1) != 0)
@@ -638,8 +706,9 @@ static void parse_line(struct _xneur_config *p, char *line)
 			if (p->popups[popup].file == NULL)
 				p->popups[popup].enabled = FALSE;
 
-			free(tmp);
-			
+			if (tmp != NULL)
+				free(tmp);
+
 			break;
 		}
 		case 33:
@@ -660,18 +729,18 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->add_space_after_autocompletion = index;
 			break;
 		}
-		case 35: // Add plugin 
+		case 35: // Add plugin
 		{
 			p->plugins->add(p->plugins, full_string);
 			break;
 		}
 		case 36: // Log Size
 		{
-			if (param != NULL) 
+			if (param != NULL)
 			  p->size_keyboard_log = atoi (param);
 			break;
 		}
-		case 37: // Log E-Mail 
+		case 37: // Log E-Mail
 		{
 			p->mail_keyboard_log = strdup (param);
 			break;
@@ -991,6 +1060,41 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->delay_send_key_apps->add(p->delay_send_key_apps, full_string);
 			break;
 		}
+		case 68: // Get Delimeters
+		{
+			KeySym symbol = XStringToKeysym(full_string);
+			if ((symbol != NoSymbol) &&
+			    (symbol != XK_BackSpace) &&
+			    (symbol != XK_Pause) &&
+			    (symbol != XK_Escape) &&
+			    (symbol != XK_Sys_Req) &&
+			    (symbol != XK_Delete) &&
+			    (symbol != XK_Return) &&
+			    (symbol != XK_Tab))
+			{
+				int is_double = FALSE;
+				for (int i = 0; i < p->delimeters_count; i++)
+				{
+					if (p->delimeters[i] == symbol)
+						is_double = TRUE;
+				}
+				if (!is_double)
+				{
+					void *tmp = realloc(p->delimeters, sizeof(KeySym) * (p->delimeters_count + 1));
+					if (tmp == NULL)
+						break;
+					p->delimeters = (KeySym *)tmp;
+					tmp = realloc(p->delimeters_string, sizeof(char) * (p->delimeters_count + 2));
+					if (tmp == NULL)
+						break;
+					p->delimeters_string = (char *)tmp;
+
+					p->delimeters[p->delimeters_count] = symbol;
+					p->delimeters_count++;
+				}
+			}
+			break;
+		}
 	}
 	free(full_string);
 }
@@ -1005,10 +1109,10 @@ static int parse_config_file(struct _xneur_config *p, const char *dir_name, cons
 	}
 
 	for (int i = 0; i < list->data_count; i++)
-	{	
+	{
 		parse_line(p, list->data[i].string);
 	}
-	
+
 	list->uninit(list);
 	return TRUE;
 }
@@ -1023,15 +1127,10 @@ static void free_structures(struct _xneur_config *p)
 	p->autocompletion_excluded_apps->uninit(p->autocompletion_excluded_apps);
 	p->dont_send_key_release_apps->uninit(p->dont_send_key_release_apps);
 	p->delay_send_key_apps->uninit(p->delay_send_key_apps);
-	
 	p->abbreviations->uninit(p->abbreviations);
-	p->plugins->uninit(p->plugins);
 	
-	for (int hotkey = 0; hotkey < MAX_HOTKEYS; hotkey++)
-	{
-		if (p->hotkeys[hotkey].key != NULL)
-			free(p->hotkeys[hotkey].key);
-	}
+	p->plugins->uninit(p->plugins);
+
 
 	for (int notify = 0; notify < MAX_NOTIFIES; notify++)
 	{
@@ -1045,22 +1144,35 @@ static void free_structures(struct _xneur_config *p)
 			free(p->popups[notify].file);
 	}
 
-	for (int action = 0; action < p->actions_count; action++)
+	if (p->actions != NULL)
 	{
-		if (p->actions[action].hotkey.key != NULL)
-			free(p->actions[action].hotkey.key);
-		if (p->actions[action].name != NULL)
-			free(p->actions[action].name);
-		if (p->actions[action].command != NULL)
-			free(p->actions[action].command);
+		for (int action = 0; action < p->actions_count; action++)
+		{
+			if (p->actions[action].hotkey.key != NULL)
+				free(p->actions[action].hotkey.key);
+		}
 	}
 
-	bzero(p->hotkeys, MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
+	if (p->user_actions != NULL)
+	{
+		for (int action = 0; action < p->user_actions_count; action++)
+		{
+			if (p->user_actions[action].hotkey.key != NULL)
+				free(p->user_actions[action].hotkey.key);
+			if (p->user_actions[action].name != NULL)
+				free(p->user_actions[action].name);
+			if (p->user_actions[action].command != NULL)
+				free(p->user_actions[action].command);
+		}
+	}
+
+	p->actions_count = 0;
+	p->user_actions_count = 0;
+
 	bzero(p->sounds, MAX_NOTIFIES * sizeof(struct _xneur_notify));
 	bzero(p->osds, MAX_NOTIFIES * sizeof(struct _xneur_notify));
 	bzero(p->popups, MAX_NOTIFIES * sizeof(struct _xneur_notify));
 
-	p->actions_count = 0;
 
 	if (p->version != NULL)
 		free(p->version);
@@ -1068,8 +1180,11 @@ static void free_structures(struct _xneur_config *p)
 	if (p->osd_font != NULL)
 		free(p->osd_font);
 
-	if (p->actions != NULL)
-		free(p->actions);
+	//if (p->actions != NULL)
+	//	free(p->actions);
+	
+	//if (p->user_actions != NULL)
+	//	free(p->user_actions);
 }
 
 static void xneur_config_reload(struct _xneur_config *p)
@@ -1085,15 +1200,21 @@ static pid_t xneur_config_set_pid(struct _xneur_config *p, pid_t process_id)
 {
 	// Set lock file to ~/.xneur/.cache/lock
 	char *lock_file_path_name = get_home_file_path_name(CACHEDIR, LOCK_NAME);
+	if (lock_file_path_name == NULL)
+		return -1;
 
 	if (process_id == 0)
 	{
-	    remove(lock_file_path_name);
+	    if (remove(lock_file_path_name) == -1)
+		{
+			free(lock_file_path_name);
+			return -1;
+		}
 		free(lock_file_path_name);
 		p->pid = process_id;
 		return process_id;
-	}     
-	
+	}
+
 	log_message(LOG, _("Saving lock file to %s (pid %d)"), lock_file_path_name, process_id);
 
 	FILE *stream = fopen(lock_file_path_name, "w");
@@ -1105,7 +1226,7 @@ static pid_t xneur_config_set_pid(struct _xneur_config *p, pid_t process_id)
 	}
 
 	free(lock_file_path_name);
-	
+
 	fprintf(stream, "%d", process_id);
 	fclose (stream);
 	p->pid = process_id;
@@ -1130,9 +1251,11 @@ static int xneur_config_get_pid(struct _xneur_config *p)
 {
 	// Get lock file from ~/.xneur/.cache/lock
 	char *config_file_path_name = get_home_file_path_name(CACHEDIR, LOCK_NAME);
+	if (config_file_path_name == NULL)
+		return -1;
 
 	log_message(LOG, _("Get lock file %s"), config_file_path_name);
-	
+
 	char *pid_str = get_file_content(config_file_path_name);
 	free(config_file_path_name);
 	if (pid_str == NULL)
@@ -1145,9 +1268,12 @@ static int xneur_config_get_pid(struct _xneur_config *p)
 		return -1;
 
 	char *ps_command = (char *) malloc(1024 * sizeof(char));
+	if (ps_command == NULL)
+		return -1;
 	snprintf(ps_command, 1024, "ps -p %d | grep xneur", process_id);
 	FILE *fp = popen(ps_command, "r");
-	free (ps_command);
+	if (ps_command != NULL)
+		free (ps_command);
 	if (fp != NULL)
 	{
 		char buffer[1024];
@@ -1159,7 +1285,7 @@ static int xneur_config_get_pid(struct _xneur_config *p)
 		}
 		pclose(fp);
 	}
-	
+
 	return -1;
 }
 
@@ -1186,12 +1312,12 @@ static void xneur_config_clear(struct _xneur_config *p)
 	p->layout_remember_apps		= list_char_init();
 	p->dont_send_key_release_apps	= list_char_init();
 	p->delay_send_key_apps	= list_char_init();
-	
+
 	p->excluded_apps		= list_char_init();
 	p->autocompletion_excluded_apps	= list_char_init();
 	p->abbreviations		= list_char_init();
 	p->plugins				= list_char_init();
-	
+
 	p->version	= NULL;
 	p->osd_font	= NULL;
 	p->actions	= NULL;
@@ -1200,6 +1326,8 @@ static void xneur_config_clear(struct _xneur_config *p)
 static int xneur_config_save(struct _xneur_config *p)
 {
  	char *config_file_path_name = get_home_file_path_name(NULL, CONFIG_NAME);
+	if (config_file_path_name == NULL)
+		return FALSE;
 
 	log_message(LOG, _("Saving main config to %s"), config_file_path_name);
 
@@ -1207,14 +1335,16 @@ static int xneur_config_save(struct _xneur_config *p)
 	if (stream == NULL)
 	{
 		log_message(ERROR, _("Can't create file %s"), config_file_path_name);
-		free(config_file_path_name);
+		if (config_file_path_name != NULL)
+			free(config_file_path_name);
 		return FALSE;
 	}
 
-	free(config_file_path_name);
+	if (config_file_path_name != NULL)
+		free(config_file_path_name);
 
 	fprintf(stream, "# It's a X Neural Switcher configuration file by XNeur\n# All values writted XNeur\n\n");
-	
+
 	fprintf(stream, "# Config version\nVersion %s\n\n", VERSION);
 	fprintf(stream, "# Work in manual mode\nManualMode %s\n\n", p->get_bool_name(p->manual_mode));
 
@@ -1226,6 +1356,14 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "#LogLevel Trace\n");
 	fprintf(stream, "LogLevel %s\n\n", p->get_log_level_name(p));
 
+	fprintf(stream, "# Word Delimeters\n");
+	fprintf(stream, "#Delimeter space\n");
+	for (int i = 0; i < p->delimeters_count; i++)
+	{
+		fprintf(stream, "Delimeter %s\n", XKeysymToString(p->delimeters[i]));
+	}
+	fprintf(stream, "\n");
+
 	fprintf(stream, "# Define unused languages\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#ExcludeLanguage de\n");
@@ -1262,30 +1400,11 @@ static int xneur_config_save(struct _xneur_config *p)
 		fprintf(stream, "SetManualApp %s\n", p->manual_apps->data[i].string);
 	fprintf(stream, "\n");
 
+	
 	fprintf(stream, "# Binds hotkeys for some actions\n");
-	for (int action = 0; action < MAX_HOTKEYS; action++)
-	{
-		fprintf(stream, "AddBind %s ", action_names[action]);
-
-		const int total_modifiers = sizeof(modifier_names) / sizeof(modifier_names[0]);
-		for (int i = 0; i < total_modifiers; i++)
-		{
-			if (p->hotkeys[action].modifiers & (1 << i))
-				fprintf(stream, "%s ", modifier_names[i]);
-		}
-
-		if (p->hotkeys[action].key != NULL)
-			fprintf(stream, "%s", p->hotkeys[action].key);
-		fprintf(stream, "\n");
-	}
-	fprintf(stream, "\n");
-
-	fprintf(stream, "# This option add user action when pressed key bind\n");
-	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#AddAction Control Alt f Firefox Browser <cmd>firefox</cmd>\n");
 	for (int action = 0; action < p->actions_count; action++)
 	{
-		fprintf(stream, "AddAction ");
+		fprintf(stream, "AddBind %s ", action_names[p->actions[action].action]);
 
 		const int total_modifiers = sizeof(modifier_names) / sizeof(modifier_names[0]);
 		for (int i = 0; i < total_modifiers; i++)
@@ -1293,10 +1412,30 @@ static int xneur_config_save(struct _xneur_config *p)
 			if (p->actions[action].hotkey.modifiers & (1 << i))
 				fprintf(stream, "%s ", modifier_names[i]);
 		}
-		if (p->actions[action].name == NULL)
-			fprintf(stream, "%s %s\n", p->actions[action].hotkey.key, p->actions[action].command);
+
+		if (p->actions[action].hotkey.key != NULL)
+			fprintf(stream, "%s", p->actions[action].hotkey.key);
+		fprintf(stream, "\n");
+	}
+	fprintf(stream, "\n");
+
+	fprintf(stream, "# This option add user action when pressed key bind\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#AddAction Control Alt f Firefox Browser <cmd>firefox</cmd>\n");
+	for (int action = 0; action < p->user_actions_count; action++)
+	{
+		fprintf(stream, "AddAction ");
+
+		const int total_modifiers = sizeof(modifier_names) / sizeof(modifier_names[0]);
+		for (int i = 0; i < total_modifiers; i++)
+		{
+			if (p->user_actions[action].hotkey.modifiers & (1 << i))
+				fprintf(stream, "%s ", modifier_names[i]);
+		}
+		if (p->user_actions[action].name == NULL)
+			fprintf(stream, "%s %s\n", p->user_actions[action].hotkey.key, p->user_actions[action].command);
 		else
-			fprintf(stream, "%s %s %s%s%s\n", p->actions[action].hotkey.key, p->actions[action].name, USR_CMD_START, p->actions[action].command, USR_CMD_END);
+			fprintf(stream, "%s %s %s%s%s\n", p->user_actions[action].hotkey.key, p->user_actions[action].name, USR_CMD_START, p->user_actions[action].command, USR_CMD_END);
 	}
 	fprintf(stream, "\n");
 
@@ -1309,7 +1448,13 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#ReplaceAbbreviation xneur X Neural Switcher\n");
 	for (int words = 0; words < p->abbreviations->data_count; words++)
-		fprintf(stream, "ReplaceAbbreviation %s\n", real_sym_to_escaped_sym(p->abbreviations->data[words].string));
+	{
+		char *str = real_sym_to_escaped_sym(p->abbreviations->data[words].string);
+		if (str == NULL)
+			continue;
+		fprintf(stream, "ReplaceAbbreviation %s\n", str);
+		free(str);
+	}
 	fprintf(stream, "\n");
 
 	fprintf(stream, "# This option enable or disable sound playing\n");
@@ -1321,7 +1466,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#SoundVolumePercent 10\n");
 	fprintf(stream, "SoundVolumePercent %d\n\n", p->volume_percent);
-	    
+
 	fprintf(stream, "# Binds sounds for some actions\n");
 	for (int sound = 0; sound < MAX_NOTIFIES; sound++)
 	{
@@ -1341,7 +1486,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#CheckSimilarWords No\n");
 	fprintf(stream, "CheckSimilarWords %s\n\n", p->get_bool_name(p->check_similar_words));
-	
+
 	fprintf(stream, "# This option enable or disable layout remember for each window\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#LayoutRememberMode No\n");
@@ -1372,10 +1517,10 @@ static int xneur_config_save(struct _xneur_config *p)
 	for (int i = 0; i < p->delay_send_key_apps->data_count; i++)
 		fprintf(stream, "DelaySendingEventApp %s\n", p->delay_send_key_apps->data[i].string);
 	fprintf(stream, "\n");
-	
+
 	fprintf(stream, "# This option define delay before sendind events to application (in milliseconds between 0 to 50).\n");
 	fprintf(stream, "SendDelay %d\n\n", p->send_delay);
-	
+
 	fprintf(stream, "# This option enable or disable logging keyboard\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#LogSave No\n");
@@ -1403,7 +1548,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#LogPort 25\n");
 	fprintf(stream, "LogPort %d\n\n", p->port_keyboard_log);
-	
+
 	fprintf(stream, "# This option enable or disable correction of iNCIDENTAL CapsLock\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#CorrectIncidentalCaps Yes\n");
@@ -1458,12 +1603,12 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#CorrectMisprint Yes\n");
 	fprintf(stream, "CorrectMisprint %s\n\n", p->get_bool_name(p->correct_misprint));
-	
+
 	fprintf(stream, "# This option enable or disable flushing internal buffer when pressed Escape\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#FlushBufferWhenPressEscape Yes\n");
 	fprintf(stream, "FlushBufferWhenPressEscape %s\n\n", p->get_bool_name(p->flush_buffer_when_press_escape));
-	
+
 	fprintf(stream, "# This option enable or disable flushing internal buffer when pressed Enter or Tab\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#FlushBufferWhenPressEnter Yes\n");
@@ -1560,7 +1705,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	for (int i = 0; i < p->dont_send_key_release_apps->data_count; i++)
 		fprintf(stream, "SetDontSendKeyReleaseApp %s\n", p->dont_send_key_release_apps->data[i].string);
 	fprintf(stream, "\n");
-	
+
 	fprintf(stream, "# Work-arround for compatibility with the completion\nCompatibilityWithCompletion %s\n\n", p->get_bool_name(p->compatibility_with_completion));
 
 	fprintf(stream, "# Disabling this option will add any application to the list of excluded applications.\nTrackingInput %s\n\n", p->get_bool_name(p->tracking_input));
@@ -1573,7 +1718,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	for (int plugins = 0; plugins < p->plugins->data_count; plugins++)
 		fprintf(stream, "LoadModule %s\n", p->plugins->data[plugins].string);
 	fprintf(stream, "\n");
-	        
+
 	fprintf(stream, "# That's all\n");
 
 	fclose(stream);
@@ -1583,12 +1728,25 @@ static int xneur_config_save(struct _xneur_config *p)
 
 static int xneur_config_replace(struct _xneur_config *p)
 {
-	char *config_file_path_name		= get_file_path_name(NULL, CONFIG_NAME);
-	char *config_backup_file_path_name	= get_file_path_name(NULL, CONFIG_BCK_NAME);
+	char *config_file_path_name = get_file_path_name(NULL, CONFIG_NAME);
+	if (config_file_path_name == NULL)
+		return FALSE;
+	char *config_backup_file_path_name = get_file_path_name(NULL, CONFIG_BCK_NAME);
+	if (config_backup_file_path_name == NULL)
+	{
+		free(config_file_path_name);
+		return FALSE;
+	}
 
 	log_message(LOG, _("Moving config file from %s to %s"), config_file_path_name, config_backup_file_path_name);
 
-	remove(config_backup_file_path_name);
+	if (remove(config_backup_file_path_name) == -1)
+	{
+		log_message(ERROR, _("Can't move file!"), config_backup_file_path_name);
+		free(config_file_path_name);
+		free(config_backup_file_path_name);
+		return FALSE;
+	}
 
 	if (rename(config_file_path_name, config_backup_file_path_name) != 0)
 	{
@@ -1616,7 +1774,8 @@ static void xneur_config_save_dict(struct _xneur_config *p, int lang)
 	char *lang_dir = (char *) malloc(path_len * sizeof(char));
 	snprintf(lang_dir, path_len, "%s/%s", LANGUAGEDIR, p->handle->languages[lang].dir);
 	save_list(p->handle->languages[lang].dictionary, lang_dir, DICT_NAME);
-	free (lang_dir);
+	if (lang_dir != NULL)
+		free (lang_dir);
 }
 
 static void xneur_config_save_pattern(struct _xneur_config *p, int lang)
@@ -1630,7 +1789,8 @@ static void xneur_config_save_pattern(struct _xneur_config *p, int lang)
 	char *lang_dir = (char *) malloc(path_len * sizeof(char));
 	snprintf(lang_dir, path_len, "%s/%s", LANGUAGEDIR, p->handle->languages[lang].dir);
 	save_list(p->handle->languages[lang].pattern, lang_dir, PATTERN_NAME);
-	free(lang_dir);
+	if (lang_dir != NULL)
+		free(lang_dir);
 }
 
 static char* xneur_config_get_lang_dir(struct _xneur_config *p, int lang)
@@ -1652,15 +1812,28 @@ static const char* xneur_config_get_log_level_name(struct _xneur_config *p)
 
 static void xneur_config_uninit(struct _xneur_config *p)
 {
+	if (p == NULL)
+		return;
+
 	free_structures(p);
 
-	free(p->hotkeys);
+	free(p->actions);
+	free(p->user_actions);
+	
 	free(p->sounds);
 	free(p->osds);
 	free(p->popups);
 
-	free(p->mail_keyboard_log);
-	free(p->host_keyboard_log);
+	if (p->delimeters != NULL)
+		free(p->delimeters);
+	if (p->delimeters_string != NULL)
+		free(p->delimeters_string);
+	p->delimeters_count = 0;
+
+	if (p->mail_keyboard_log != NULL) 
+		free(p->mail_keyboard_log);
+	if (p->host_keyboard_log != NULL) 
+		free(p->host_keyboard_log);
 
 	xneur_handle_destroy(p->handle);
 	
@@ -1673,14 +1846,14 @@ struct _xneur_config* xneur_config_init(void)
 	bzero(p, sizeof(struct _xneur_config));
 
 	p->pid = -1;
-	
+
 	p->handle = xneur_handle_create();
 
-	//char *a = "ghbdtn";
-	//xneur_get_layout(p->handle, a);
-	
-	p->hotkeys = (struct _xneur_hotkey *) malloc(MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
-	bzero(p->hotkeys, MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
+	p->delimeters = (KeySym *) malloc(sizeof(KeySym));
+	p->delimeters_string = (char *) malloc(sizeof(char));
+	p->delimeters[p->delimeters_count] = XK_space;
+	p->delimeters_string[p->delimeters_count] = '\0';
+	p->delimeters_count++;
 
 	p->sounds = (struct _xneur_notify *) malloc(MAX_NOTIFIES * sizeof(struct _xneur_notify));
 	bzero(p->sounds, MAX_NOTIFIES * sizeof(struct _xneur_notify));
@@ -1694,7 +1867,7 @@ struct _xneur_config* xneur_config_init(void)
 	p->mail_keyboard_log = NULL;
 	p->host_keyboard_log = NULL;
 	p->port_keyboard_log = 25;
-	
+
 	p->log_level			= LOG;
 	p->excluded_apps		= list_char_init();
 	p->auto_apps			= list_char_init();
@@ -1702,26 +1875,26 @@ struct _xneur_config* xneur_config_init(void)
 	p->layout_remember_apps		= list_char_init();
 	p->dont_send_key_release_apps = list_char_init();
 	p->delay_send_key_apps	= list_char_init();
-	
+
 	p->window_layouts		= list_char_init();
 	p->abbreviations		= list_char_init();
 	p->autocompletion_excluded_apps	= list_char_init();
 	p->plugins		= list_char_init();
 
 	p->check_similar_words = TRUE;
-	
-	p->troubleshoot_backspace = FALSE; 
-	p->troubleshoot_left_arrow = FALSE; 
+
+	p->troubleshoot_backspace = FALSE;
+	p->troubleshoot_left_arrow = FALSE;
 	p->troubleshoot_right_arrow = FALSE;
 	p->troubleshoot_up_arrow = FALSE;
 	p->troubleshoot_down_arrow = FALSE;
 	p->troubleshoot_delete = FALSE;
 	p->troubleshoot_switch = TRUE;
 	p->troubleshoot_full_screen = TRUE;
-	
+
 	p->tracking_input = TRUE;
 	p->tracking_mouse = TRUE;
-	
+
 	p->popup_expire_timeout = 1000;
 
 	// Function mapping
@@ -1748,4 +1921,3 @@ struct _xneur_config* xneur_config_init(void)
 
 	return p;
 }
-
