@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2013 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -29,10 +29,12 @@
 #define CACHEDIR			".cache"
 #define LOCK_NAME			"lock"
 
-#define MAX_NOTIFIES		38
-#define MAX_HOTKEYS			24
+#define MAX_NOTIFIES		37
+#define MAX_HOTKEYS			23
 
-#include <sys/types.h>	
+#include <sys/types.h>
+#define XK_PUBLISHING
+#include <X11/XKBlib.h>
 #include "xneur.h"
 
 enum _flag_action
@@ -69,7 +71,7 @@ enum _notify_action
 	NOTIFY_ENABLE_LAYOUT_0,
 	NOTIFY_ENABLE_LAYOUT_1,
 	NOTIFY_ENABLE_LAYOUT_2,
-	NOTIFY_ENABLE_LAYOUT_3,
+	NOTIFY_ENABLE_LAYOUT_3 = 10,
 	NOTIFY_AUTOMATIC_CHANGE_WORD,
 	NOTIFY_MANUAL_CHANGE_WORD,
 	NOTIFY_MANUAL_TRANSLIT_WORD,
@@ -79,7 +81,7 @@ enum _notify_action
 	NOTIFY_CHANGE_SELECTED,
 	NOTIFY_TRANSLIT_SELECTED,
 	NOTIFY_CHANGECASE_SELECTED,
-	NOTIFY_PREVIEW_CHANGE_SELECTED,
+	NOTIFY_PREVIEW_CHANGE_SELECTED = 20,
 	NOTIFY_CHANGE_CLIPBOARD,
 	NOTIFY_TRANSLIT_CLIPBOARD,
 	NOTIFY_CHANGECASE_CLIPBOARD,
@@ -89,15 +91,13 @@ enum _notify_action
 	NOTIFY_CORR_TWO_CAPITAL_LETTER,
 	NOTIFY_CORR_TWO_SPACE,
 	NOTIFY_CORR_TWO_MINUS,
-	NOTIFY_CORR_COPYRIGHT,
+	NOTIFY_CORR_COPYRIGHT = 30,
 	NOTIFY_CORR_TRADEMARK,
 	NOTIFY_CORR_REGISTERED,
 	NOTIFY_CORR_ELLIPSIS,
 	NOTIFY_CORR_MISPRINT,
 	NOTIFY_EXEC_USER_ACTION,
-	NOTIFY_BLOCK_EVENTS,
-	NOTIFY_UNBLOCK_EVENTS,
-	NOTIFY_NONE,
+	NOTIFY_NONE = 36,
 };
 
 enum _hotkey_action
@@ -112,7 +112,7 @@ enum _hotkey_action
 	ACTION_CHANGECASE_SELECTED,
 	ACTION_PREVIEW_CHANGE_SELECTED,
 	ACTION_CHANGE_CLIPBOARD,
-	ACTION_TRANSLIT_CLIPBOARD,
+	ACTION_TRANSLIT_CLIPBOARD = 10,
 	ACTION_CHANGECASE_CLIPBOARD,
 	ACTION_PREVIEW_CHANGE_CLIPBOARD,
 	ACTION_ENABLE_LAYOUT_0,
@@ -122,11 +122,10 @@ enum _hotkey_action
 	ACTION_ROTATE_LAYOUT,
 	ACTION_ROTATE_LAYOUT_BACK,
 	ACTION_REPLACE_ABBREVIATION,
-	ACTION_AUTOCOMPLETION,
+	ACTION_AUTOCOMPLETION = 20,
 	ACTION_ROTATE_AUTOCOMPLETION,
-	ACTION_BLOCK_EVENTS,
 	ACTION_INSERT_DATE,
-	ACTION_NONE,
+	ACTION_NONE=23,
 };
 
 enum _change_action
@@ -166,6 +165,7 @@ struct _xneur_hotkey
 	char *key;
 };
 
+
 struct _xneur_notify
 {
 	char *file;
@@ -173,6 +173,12 @@ struct _xneur_notify
 };
 
 struct _xneur_action
+{
+	struct _xneur_hotkey hotkey;
+	enum _hotkey_action action;
+};
+
+struct _xneur_user_action
 {
 	struct _xneur_hotkey hotkey;
 	char *name;
@@ -184,7 +190,7 @@ struct _xneur_config
 	char *version;
 
 	pid_t pid;
-	
+
 	void (*get_library_version) (int *major_version, int *minor_version); // This function MUST be first
 
 	struct _list_char *excluded_apps;
@@ -193,24 +199,31 @@ struct _xneur_config
 	struct _list_char *layout_remember_apps;
 	struct _list_char *dont_send_key_release_apps;
 	struct _list_char *delay_send_key_apps;
-	
+
 	struct _list_char *window_layouts;
 	struct _list_char *abbreviations;
 	struct _list_char *plugins;
 
 	struct _xneur_handle *handle;		// Array of languages used in program
-	struct _xneur_hotkey *hotkeys;			// Array of hotkeys used in program
+	
 	struct _xneur_notify *sounds;			// Array of sounds for actions
 	struct _xneur_notify *osds;			// Array of OSDs for actions
 	struct _xneur_notify *popups;			// Array of popups for actions
-	struct _xneur_action *actions;			// Array of actions
+	
+	struct _xneur_action *actions;			// Array of hotkeys used in program
+	int actions_count;				// Count of hotkeys
 
-	int   actions_count;				// Count of actions
+	struct _xneur_user_action *user_actions;			// Array of actions
+	int user_actions_count;				// Count of actions
+
+	KeySym *delimeters;
+	char *delimeters_string;
+	int delimeters_count;
 
 	int   manual_mode;				// Enable manual processing mode
 	int   log_level;				// Maximum level of log messages to print
 	int   send_delay;				// Delay before send event (in milliseconds)
-	
+
 	int   default_group;				// Initial keyboard layout for all new applications
 
 	int   play_sounds;				// Play sound samples or not
@@ -223,7 +236,7 @@ struct _xneur_config
 	int   rotate_layout_after_convert;	// Rotate layout after convert selected text
 
 	int   correct_misprint;
-	
+
 	int   correct_incidental_caps;			// Change iNCIDENTAL CapsLock
 	int   correct_two_capital_letter;		// Change two CApital letter
 	int   correct_space_with_punctuation;	// Correct spaces before punctuation
@@ -235,14 +248,12 @@ struct _xneur_config
 	int   correct_tm_with_trademark;
 	int   correct_r_with_registered;
 	int   correct_three_points_with_ellipsis;
-	
+
 	int   flush_buffer_when_press_escape;		// Flush internal buffer when pressed Escape
 	int   flush_buffer_when_press_enter;		// Flush internal buffer when pressed Enter or Tab
 	int   check_lang_on_process;			// Check lang on input process
 	int   disable_capslock;				// Disable CapsLock use
 
-	int   block_events;					// Block all events (keyboard and mouse)
-	
 	int   autocompletion;			// Save pattern and mining
 	int   add_space_after_autocompletion;
 	struct _list_char *autocompletion_excluded_apps;
@@ -271,12 +282,12 @@ struct _xneur_config
 	int troubleshoot_full_screen;
 	int troubleshoot_enter;		// Don't correct word when pressed Enter
 	int troubleshoot_tab;		// Don't correct word when pressed Tab
-	
+
 	int compatibility_with_completion;
-		
+
 	int tracking_input;
 	int tracking_mouse;
-		
+
 	char* (*get_home_dict_path) (const char *dir_name, const char *file_name);
 	char* (*get_global_dict_path) (const char *dir_name, const char *file_name);
 	const char* (*get_bool_name) (int option);
