@@ -22,6 +22,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <errno.h>
 
 #include "types.h"
 #include "list_char.h"
@@ -43,15 +44,25 @@ static int get_max_path_len(void)
 
 char* get_file_content(const char *file_name)
 {
-	struct stat sb;
-
-	if (stat(file_name, &sb) != 0 || sb.st_size < 0)
+	if (file_name == NULL)
 		return NULL;
-
+	
 	FILE *stream = fopen(file_name, "rb");
 	if (stream == NULL)
 		return NULL;
 
+	struct stat sb;
+	if (stat(file_name, &sb) != 0)
+	{
+		fclose(stream);
+		return NULL;
+	}
+	if (sb.st_size < 0)
+	{
+		fclose(stream);
+		return NULL;
+	}
+	
 	unsigned int file_len = sb.st_size;
 
 	char *content = (char *) malloc((file_len + 2) * sizeof(char)); // + 1 '\0'
@@ -119,36 +130,58 @@ char* get_file_path_name(const char *dir_name, const char *file_name)
 
 char* get_home_file_path_name(const char *dir_name, const char *file_name)
 {
+	mode_t mode = S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
 	int max_path_len = get_max_path_len();
-
 	char *path_file = (char *) malloc((max_path_len + 1) * sizeof(char));
 
 	if (dir_name == NULL)
 	{
 		snprintf(path_file, max_path_len, "%s/%s", getenv("HOME"), HOME_CONF_DIR);
-		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
-
+		if (mkdir(path_file, mode) != 0 && errno != EEXIST)
+		{
+			free(path_file);
+			return NULL;
+		}
 		snprintf(path_file, max_path_len, "%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, file_name);
 	}
 	else
 	{
 		snprintf(path_file, max_path_len, "%s/%s", getenv("HOME"), HOME_CONF_DIR);
-		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
-		
+		if (mkdir(path_file, mode) != 0 && errno != EEXIST)
+		{
+			free(path_file);
+			return NULL;
+		}
 		char *dir = strdup(dir_name);
 		char *dir_part = strsep(&dir, DIR_SEPARATOR);
 		snprintf(path_file, max_path_len, "%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, dir_part);
-		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+		if (mkdir(path_file, mode) != 0 && errno != EEXIST)
+		{
+			free(path_file);
+			free(dir_part);
+			free(dir);
+			return NULL;
+		}
 		while (dir != NULL)
 		{
 			path_file = strcat(path_file, DIR_SEPARATOR); 
 			char *dir_part = strsep(&dir, DIR_SEPARATOR);
 			path_file = strcat(path_file, dir_part);
-			mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+			if (mkdir(path_file, mode) != 0 && errno != EEXIST)
+			{
+				free(path_file);
+				free(dir_part);
+				free(dir);
+				return NULL;
+			}
 		}
 		free(dir_part);
 		
-		mkdir(path_file, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
+		if (mkdir(path_file, mode) != 0 && errno != EEXIST)
+		{
+			free(path_file);
+			return NULL;
+		}
 		snprintf(path_file, max_path_len, "%s/%s/%s/%s", getenv("HOME"), HOME_CONF_DIR, dir_name, file_name);
 	}
 	return path_file;
@@ -157,9 +190,7 @@ char* get_home_file_path_name(const char *dir_name, const char *file_name)
 struct _list_char* load_list(const char *dir_name, const char *file_name, int sort_flag)
 {
 	struct _list_char *list = list_char_init();
-	
 	char *file_path_name = get_file_path_name(dir_name, file_name);
-
 	char *content = get_file_content(file_path_name);
 	if (content == NULL)
 	{
