@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2013 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -111,12 +111,15 @@ static long get_next_property_value (unsigned char **pointer, long *length, int
 struct _xneur_handle *xneur_handle_create (void)
 {
 	struct _xneur_handle *handle = (struct _xneur_handle *) malloc(sizeof(struct _xneur_handle));;
-
+	if (handle == NULL)
+		return NULL;
+	
 	XkbDescPtr kbd_desc_ptr;
 
 	if (!(kbd_desc_ptr = XkbAllocKeyboard()))
 	{
-		free(handle);
+		if (handle != NULL)
+			free(handle);
 		return NULL;
 	}
 	
@@ -137,7 +140,8 @@ struct _xneur_handle *xneur_handle_create (void)
 	{
 		XCloseDisplay(display);
 		XkbFreeKeyboard(kbd_desc_ptr, XkbAllComponentsMask, True);
-		free(handle);
+		if (handle != NULL)
+			free(handle);
 		return NULL;
 	}
 
@@ -152,7 +156,8 @@ struct _xneur_handle *xneur_handle_create (void)
 	{
 		XCloseDisplay(display);
 		XkbFreeKeyboard(kbd_desc_ptr, XkbAllComponentsMask, True);
-		free(handle);
+		if (handle != NULL)
+			free(handle);
 		return NULL;
 	}
 
@@ -161,7 +166,8 @@ struct _xneur_handle *xneur_handle_create (void)
 	{
 		XCloseDisplay(display);
 		XkbFreeKeyboard(kbd_desc_ptr, XkbAllComponentsMask, True);
-		free(handle);
+		if (handle != NULL)
+			free(handle);
 		return NULL;
 	}
 	Window rw = RootWindow(display, DefaultScreen(display));
@@ -181,7 +187,8 @@ struct _xneur_handle *xneur_handle_create (void)
 	{
 		XCloseDisplay(display);
 		XkbFreeKeyboard(kbd_desc_ptr, XkbAllComponentsMask, True);
-		free(handle);
+		if (handle != NULL)
+			free(handle);
 		return NULL;
 	}
 	
@@ -197,7 +204,8 @@ struct _xneur_handle *xneur_handle_create (void)
 	{
 		XCloseDisplay(display);
 		XkbFreeKeyboard(kbd_desc_ptr, XkbAllComponentsMask, True);
-		free(handle);
+		if (handle != NULL)
+			free(handle);
 		return NULL;
 	}
 	
@@ -211,7 +219,8 @@ struct _xneur_handle *xneur_handle_create (void)
 		{
 			XCloseDisplay(display);
 			XkbFreeKeyboard(kbd_desc_ptr, XkbAllComponentsMask, True);
-			free(handle);
+			if (handle != NULL)
+				free(handle);
 			return NULL;
 		}
 
@@ -228,7 +237,8 @@ struct _xneur_handle *xneur_handle_create (void)
 	{
 		XCloseDisplay(display);
 		XkbFreeKeyboard(kbd_desc_ptr, XkbAllComponentsMask, True);
-		free(handle);
+		if (handle != NULL)
+			free(handle);
 		return NULL;
 	}
 	//log_message(ERROR, "%s", 
@@ -246,13 +256,19 @@ struct _xneur_handle *xneur_handle_create (void)
 		//log_message (ERROR, "%s", group_name);
 		char *short_name = strsep(&prop_value, ",");
 		//log_message (ERROR, "%s", short_name);
-		
-		handle->languages = (struct _xneur_language *) realloc(handle->languages, (handle->total_languages + 1) * sizeof(struct _xneur_language));
+
+		void *tmp = realloc(handle->languages, (handle->total_languages + 1) * sizeof(struct _xneur_language));
+		if (tmp == NULL)
+			continue;
+		handle->languages = (struct _xneur_language *) tmp;
 		bzero(&(handle->languages[handle->total_languages]), sizeof(struct _xneur_language));
 
 		handle->languages[handle->total_languages].name	= strdup(group_name);
 		handle->languages[handle->total_languages].dir	= strdup(short_name);
-		handle->languages[handle->total_languages].dir	= (char *) realloc (handle->languages[handle->total_languages].dir, sizeof(char) * 3); // 'xx' + '\0'
+		tmp = realloc (handle->languages[handle->total_languages].dir, sizeof(char) * 3); // 'xx' + '\0'
+		if (tmp == NULL)
+			continue;
+		handle->languages[handle->total_languages].dir	= (char *)tmp;
 		handle->languages[handle->total_languages].dir[2] = NULLSYM;
 		handle->languages[handle->total_languages].group	= group;
 		handle->languages[handle->total_languages].excluded	= FALSE;
@@ -274,20 +290,21 @@ struct _xneur_handle *xneur_handle_create (void)
 		free(handle);
 		return NULL;
 	}
+
 #ifdef WITH_ASPELL
 	// init aspell spellers
 	handle->spell_checkers = (AspellSpeller **) malloc(handle->total_languages * sizeof(AspellSpeller*));
-	handle->has_spell_checker = (int *) malloc(handle->total_languages * sizeof(int *));
+	handle->has_spell_checker = (int *) malloc(handle->total_languages * sizeof(int));
 	handle->spell_config = new_aspell_config();
 #endif
 
 #ifdef WITH_ENCHANT
 	// init enchant brocker and dicts
 	handle->enchant_dicts = (EnchantDict **) malloc(handle->total_languages * sizeof(EnchantDict *));
-	handle->has_enchant_checker = (int *) malloc(handle->total_languages * sizeof(int *));
+	handle->has_enchant_checker = (int *) malloc(handle->total_languages * sizeof(int));
 	handle->enchant_broker = enchant_broker_init ();
 #endif
-	
+
 	for (int lang = 0; lang < handle->total_languages; lang++)
 	{
 		int path_len = strlen(LANGUAGEDIR) + strlen(handle->languages[lang].dir) + 2;
@@ -320,7 +337,6 @@ struct _xneur_handle *xneur_handle_create (void)
 
 		if (lang_dir != NULL)
 			free(lang_dir);
-
 	}
 
 #ifdef WITH_ASPELL
@@ -375,8 +391,9 @@ struct _xneur_handle *xneur_handle_create (void)
 		if (j != names_len)
 		{
 			handle->enchant_dicts[lang] = NULL;
-			char *dict_name = NULL;
-			dict_name = malloc (2 * strlen(spell_names[j]) + 2);
+			char *dict_name = malloc (2 * strlen(spell_names[j]) + 2);
+			if (dict_name == NULL)
+				continue;
 			dict_name[0] = NULLSYM;
 			strcat(dict_name, spell_names[j]);
 			strcat(dict_name, "_");
@@ -391,7 +408,8 @@ struct _xneur_handle *xneur_handle_create (void)
 				if (enchant_broker_dict_exists(handle->enchant_broker, dict_name) == FALSE)
 				{
 					handle->has_enchant_checker[lang] = 0;
-					free(dict_name);
+					if (dict_name != NULL)
+						free(dict_name);
 					continue;
 				}
 			}
@@ -399,8 +417,9 @@ struct _xneur_handle *xneur_handle_create (void)
 			//printf("   [!] Loaded dict %s\n", dict_name);
 			handle->enchant_dicts[lang] = enchant_broker_request_dict (handle->enchant_broker, dict_name);
 			handle->has_enchant_checker[lang] = 1;
-			
-			free(dict_name);
+
+			if (dict_name != NULL)
+				free(dict_name);
 		}
 		else
 		{
@@ -424,7 +443,8 @@ struct _xneur_handle *xneur_handle_create (void)
 #endif
 			);
 	}
-	return handle;
+
+	return handle;	
 }
 
 void xneur_handle_destroy (struct _xneur_handle *handle)
@@ -443,6 +463,9 @@ void xneur_handle_destroy (struct _xneur_handle *handle)
 		if ((handle->enchant_dicts[lang] != NULL) && (handle->has_enchant_checker[lang]))
 			enchant_broker_free_dict (handle->enchant_broker, handle->enchant_dicts[lang]);
 #endif
+
+		if (handle->languages == NULL)
+			continue;
 		
 		if (handle->languages[lang].temp_dictionary != NULL)
 			handle->languages[lang].temp_dictionary->uninit(handle->languages[lang].temp_dictionary);
@@ -465,22 +488,26 @@ void xneur_handle_destroy (struct _xneur_handle *handle)
 			free(handle->languages[lang].dir);
 	}
 	handle->total_languages = 0;
-	if (handle->languages != NULL)
-		free(handle->languages);
+	free(handle->languages);
 		
 #ifdef WITH_ASPELL
 	delete_aspell_config(handle->spell_config);
-	free(handle->spell_checkers);
-	free(handle->has_spell_checker);
+	if (handle->spell_checkers != NULL)
+		free(handle->spell_checkers);
+	if (handle->has_spell_checker != NULL)
+		free(handle->has_spell_checker);
 #endif
 	
 #ifdef WITH_ENCHANT
 	enchant_broker_free (handle->enchant_broker);
-	free(handle->enchant_dicts);
-	free(handle->has_enchant_checker);
+	if (handle->enchant_dicts != NULL)
+		free(handle->enchant_dicts);
+	if (handle->has_enchant_checker != NULL)
+		free(handle->has_enchant_checker);
 #endif
 	
 	free(handle);
+
 }
 
 int xneur_get_layout (struct _xneur_handle *handle, char *word)
