@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2013 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -31,6 +31,7 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <time.h>
+#include <assert.h>
 
 #include "xnconfig.h"
 #include "xnconfig_files.h"
@@ -60,11 +61,20 @@ time_t last_log_time = 0;
 
 // Private
 static void set_new_size(struct _buffer *p, int new_size)
-{
+{	
+	char *tmp_char = (char *) realloc(p->content, new_size * sizeof(char));
+	if (tmp_char == NULL)
+		return;
+	p->content = tmp_char;
+	KeyCode *tmp_kc = (KeyCode *) realloc(p->keycode, new_size * sizeof(KeyCode));
+	if (tmp_kc == NULL)
+		return;
+	p->keycode = tmp_kc;
+	int *tmp_int = (int *) realloc(p->keycode_modifiers, new_size * sizeof(int));
+	if (tmp_int == NULL)
+		return;
+	p->keycode_modifiers = tmp_int;
 	p->cur_size		= new_size;
-	p->content		= (char *) realloc(p->content, p->cur_size * sizeof(char));
-	p->keycode		= (KeyCode *) realloc(p->keycode, p->cur_size * sizeof(KeyCode));
-	p->keycode_modifiers	= (int *) realloc(p->keycode_modifiers, p->cur_size * sizeof(int));
 }
 
 static void buffer_set_lang_mask(struct _buffer *p, int lang)
@@ -96,24 +106,47 @@ static void buffer_set_caps_mask(struct _buffer *p)
 
 static void buffer_mail_and_archive(char *file_path_name)
 {
+	if (file_path_name == NULL)
+		return;
+	
 	time_t curtime = time(NULL);
 	struct tm *loctime = localtime(&curtime);
 	if (loctime == NULL)
 		return;
 	
 	char *date = malloc(256 * sizeof(char));
+	if (date == NULL)
+		return;
 	char *time = malloc(256 * sizeof(char));
+	if (time == NULL)
+	{
+		free(date);
+		return;
+	}
 	strftime(date, 256, "%x", loctime);
 	strftime(time, 256, "%X", loctime);
 	
 	int len = strlen(file_path_name) + strlen(date) + strlen(time) + 4;
 	char *arch_file_path_name = malloc(len * sizeof (char));
+	if (arch_file_path_name == NULL)
+	{
+		free(date);
+		free(time);
+		return;
+	}
 	snprintf(arch_file_path_name, len, "%s %s %s", file_path_name, date, time);
 		
 	if (rename(file_path_name, arch_file_path_name) == 0)
 	{
 		// Compress the file
 		char *gz_arch_file_path_name = malloc(len+3 * sizeof (char));
+		if (gz_arch_file_path_name == NULL)
+		{
+			free(date);
+			free(time);
+			free(arch_file_path_name);
+			return;
+		}
 		snprintf(gz_arch_file_path_name, len+3, "%s%s", arch_file_path_name, ".gz");
 	
 		FILE *source = fopen(arch_file_path_name, "r");
@@ -126,7 +159,15 @@ static void buffer_mail_and_archive(char *file_path_name)
 			fclose (dest);
 
 		// Remove uncompressed file
-		remove(arch_file_path_name);
+		if (remove(arch_file_path_name) == -1)
+		{
+			free(file_path_name);
+			free(time);
+			free(date);
+			free(arch_file_path_name);
+			free(gz_arch_file_path_name);
+			return;
+		}
 
 		log_message(DEBUG, _("Compressed old log file to %s"), gz_arch_file_path_name);
 
@@ -134,20 +175,15 @@ static void buffer_mail_and_archive(char *file_path_name)
 		send_mail_with_attach(gz_arch_file_path_name, xconfig->host_keyboard_log, xconfig->port_keyboard_log, xconfig->mail_keyboard_log);
 		log_message(DEBUG, _("Sended log to e-mail %s via %s:%d host"), xconfig->mail_keyboard_log, xconfig->host_keyboard_log, xconfig->port_keyboard_log);
 
-		if (gz_arch_file_path_name != NULL) 
-			free(gz_arch_file_path_name);
+		free(gz_arch_file_path_name);
 	}
 	else
 		log_message (ERROR, _("Error rename file \"%s\" to \"%s\""), file_path_name, arch_file_path_name);
-
-	if (arch_file_path_name != NULL)
-		free(arch_file_path_name);
-	if (file_path_name != NULL)
-		free(file_path_name);
-	if (time != NULL)
-		free(time);
-	if (date != NULL)
-		free(date);
+		
+	free(file_path_name);
+	free(time);
+	free(date);
+	free(arch_file_path_name);
 }
 
 static void buffer_save(struct _buffer *p, char *file_name, Window window)
@@ -171,30 +207,23 @@ static void buffer_save(struct _buffer *p, char *file_name, Window window)
 		return;
 	
 	char *file_path_name = get_home_file_path_name(NULL, file_name);
-
+	if (file_path_name == NULL)
+		return;
+	
 	time_t curtime = time(NULL);
 	struct tm *loctime = localtime(&curtime);
 	if (loctime == NULL)
+	{
+		free(file_path_name);
 		return;
+	}
 	
 	char *buffer = malloc(256 * sizeof(char));
-	
-	// Check file size
-	struct stat sb;
-
-	if (stat(file_path_name, &sb) == 0 && sb.st_size > xconfig->size_keyboard_log)
+	if (buffer == NULL)
 	{
-		pthread_attr_t mail_and_archive_thread_attr;
-		pthread_attr_init(&mail_and_archive_thread_attr);
-		pthread_attr_setdetachstate(&mail_and_archive_thread_attr, PTHREAD_CREATE_DETACHED);
-
-		pthread_t mail_and_archive_thread;
-		char *thread_file = strdup (file_path_name);
-		pthread_create(&mail_and_archive_thread, &mail_and_archive_thread_attr, (void*) &buffer_mail_and_archive, thread_file);
-
-		pthread_attr_destroy(&mail_and_archive_thread_attr);
+		free(file_path_name);
+		return;
 	}
-	//
 
 	// Check existing log file
 	FILE *stream = fopen(file_path_name, "r");
@@ -208,10 +237,32 @@ static void buffer_save(struct _buffer *p, char *file_name, Window window)
 			return;
 		}
 		fprintf(stream, "<html><head><meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\"><title>X Neural Switcher Log</title></head><body>\n");
-		fprintf(stream, "<ul></body></html>\n");
+		fprintf(stream, "<ul></ul></body></html>\n");
 	}
+	else
+	{
+		// Check file size
+		struct stat sb;
+		if (stat(file_path_name, &sb) == 0)
+		{
+			if (sb.st_size > xconfig->size_keyboard_log)
+			{
+				pthread_attr_t mail_and_archive_thread_attr;
+				pthread_attr_init(&mail_and_archive_thread_attr);
+				pthread_attr_setdetachstate(&mail_and_archive_thread_attr, PTHREAD_CREATE_DETACHED);
+
+				pthread_t mail_and_archive_thread;
+				char *thread_file = strdup (file_path_name);
+				pthread_create(&mail_and_archive_thread, &mail_and_archive_thread_attr, (void*) &buffer_mail_and_archive, thread_file);
+
+				pthread_attr_destroy(&mail_and_archive_thread_attr);
+			}
+		}
+	}
+	//
 	fclose (stream);
 	
+	
 	stream = fopen(file_path_name, "r+");
 	free(file_path_name);
 	if (stream == NULL)
@@ -220,8 +271,13 @@ static void buffer_save(struct _buffer *p, char *file_name, Window window)
 		return;
 	}
 
-	fseek(stream, -15, SEEK_END);
-
+	if (fseek(stream, -20, SEEK_END) == -1)
+	{
+		free(buffer);
+		fclose(stream);
+		return;
+	}
+	
 	bzero(buffer, 256 * sizeof(char));
 	strftime(buffer, 256, "%x", loctime);
 
@@ -242,7 +298,8 @@ static void buffer_save(struct _buffer *p, char *file_name, Window window)
 		strftime(buffer, 256, "%X", loctime);
 		fprintf(stream, "</ul><ul>\n<font color=\"#0000FF\" size=\"2\">(%s): </font>", buffer);
 	}
-	free(buffer);
+	if (buffer != NULL)
+		free(buffer);
 	
 	for (int i = 0; i < p->cur_pos; i++)
 	{
@@ -268,7 +325,7 @@ static void buffer_save(struct _buffer *p, char *file_name, Window window)
 			fprintf(stream, "&nbsp;");
 		else	
 			fprintf(stream, "%s", symbol);
-		
+
 		free(symbol);
 	}
 
@@ -289,9 +346,13 @@ static void buffer_clear(struct _buffer *p)
 
 	for (int i=0; i<p->handle->total_languages; i++)
 	{
-		p->i18n_content[i].content = realloc(p->i18n_content[i].content, sizeof(char));
+		char *tmp = realloc(p->i18n_content[i].content, sizeof(char));
+		if (tmp != NULL)
+			p->i18n_content[i].content = tmp;
 		p->i18n_content[i].content[0] = NULLSYM;
-		p->i18n_content[i].content_unchanged = realloc(p->i18n_content[i].content_unchanged, sizeof(char));
+		tmp = realloc(p->i18n_content[i].content_unchanged, sizeof(char));
+		if (tmp != NULL)
+			p->i18n_content[i].content_unchanged = tmp;
 		p->i18n_content[i].content_unchanged[0] = NULLSYM;
 	}
 }
@@ -334,18 +395,29 @@ static void buffer_set_i18n_content(struct _buffer *p)
 			
 			char *symbol_unchanged = p->keymap->keycode_to_symbol(p->keymap, p->keycode[k], i, modifier);
 			if (symbol_unchanged == NULL)
+			{
+				free(symbol);
 				continue;
+			}
 			
-			p->i18n_content[i].content = (char *) realloc(p->i18n_content[i].content, (strlen(p->i18n_content[i].content) + strlen(symbol) + 1) * sizeof(char));
+			char *tmp = (char *) realloc(p->i18n_content[i].content, (strlen(p->i18n_content[i].content) + strlen(symbol) + 1) * sizeof(char));
+			assert(tmp != NULL);
+			p->i18n_content[i].content = tmp;
 			p->i18n_content[i].content = strcat(p->i18n_content[i].content, symbol);
 
-			p->i18n_content[i].content_unchanged = (char *) realloc(p->i18n_content[i].content_unchanged, (strlen(p->i18n_content[i].content_unchanged) + strlen(symbol_unchanged) + 1) * sizeof(char));
+			tmp = (char *) realloc(p->i18n_content[i].content_unchanged, (strlen(p->i18n_content[i].content_unchanged) + strlen(symbol_unchanged) + 1) * sizeof(char));
+			assert(tmp != NULL);
+			p->i18n_content[i].content_unchanged = tmp;
 			p->i18n_content[i].content_unchanged = strcat(p->i18n_content[i].content_unchanged, symbol_unchanged);
 
-			p->i18n_content[i].symbol_len = (int *) realloc(p->i18n_content[i].symbol_len, (k + 1) * sizeof(int));
+			tmp = (char *)realloc(p->i18n_content[i].symbol_len, (k + 1) * sizeof(int));
+			assert(tmp != NULL);
+			p->i18n_content[i].symbol_len = (int *) tmp;
 			p->i18n_content[i].symbol_len[k] = strlen(symbol);
 
-			p->i18n_content[i].symbol_len_unchanged = (int *) realloc(p->i18n_content[i].symbol_len_unchanged, (k + 1) * sizeof(int));
+			tmp = (char *)realloc(p->i18n_content[i].symbol_len_unchanged, (k + 1) * sizeof(int));
+			assert(tmp != NULL);
+			p->i18n_content[i].symbol_len_unchanged = (int *)tmp; 
 			p->i18n_content[i].symbol_len_unchanged[k] = strlen(symbol_unchanged);
 
 			free(symbol);
@@ -356,25 +428,22 @@ static void buffer_set_i18n_content(struct _buffer *p)
 
 static void buffer_set_content(struct _buffer *p, const char *new_content)
 {
+	if (new_content == NULL)
+		return;
+	char *content = strdup(new_content);
+	if (content == NULL)
+		return;
+	
 	p->clear(p);
-	char *content = NULL;
-	content = strdup(new_content);
+	
 	p->cur_pos = strlen(content);
 	if (p->cur_pos >= p->cur_size)
 		set_new_size(p, p->cur_pos + 1);
 
-	if (p->content == NULL || p->keycode == NULL || p->keycode_modifiers == NULL)
-	{
-		if (content != NULL)
-			free(content);
-		return;
-	}
-	
 	p->content[p->cur_pos] = NULLSYM;
 	if (!p->cur_pos)
 	{
-		if (p->content != NULL)
-			free(content);
+		free(content);
 		return;
 	}
 
@@ -386,18 +455,6 @@ static void buffer_set_content(struct _buffer *p, const char *new_content)
 	p->cur_pos = strlen(p->content);
 	set_new_size(p, p->cur_pos + 1);
 
-	if (p->content == NULL || p->keycode == NULL || p->keycode_modifiers == NULL)
-	{
-		for (int i = 0; i < p->handle->total_languages; i++)
-		{
-			free(p->i18n_content[i].content);
-			free(p->i18n_content[i].symbol_len);
-			free(p->i18n_content[i].content_unchanged);
-			free(p->i18n_content[i].symbol_len_unchanged);
-		}
-		return;
-	}
-	
 	buffer_set_i18n_content(p);
 }
 
@@ -425,19 +482,22 @@ static void buffer_change_case(struct _buffer *p)
 		int nbytes = XLookupString((XKeyEvent *) &event, symbol, 256, NULL, NULL);
 		if (nbytes <= 0)
 			continue;
-
+		if (symbol == NULL)
+			continue;
+		
 		symbol[nbytes] = NULLSYM;	
 
 		if (ispunct(symbol[0]) || isdigit(symbol[0]))
 			continue;
-
+		
 		if (p->keycode_modifiers[i] & ShiftMask)
 			p->keycode_modifiers[i] = (p->keycode_modifiers[i] & ~ShiftMask);
 		else
 			p->keycode_modifiers[i] = (p->keycode_modifiers[i] | ShiftMask);
 	}
-	
-	free(symbol);
+
+	if (symbol != NULL)
+		free(symbol);
 	XCloseDisplay(display);
 }
 
@@ -476,7 +536,7 @@ static void buffer_add_symbol(struct _buffer *p, char sym, KeyCode keycode, int
 	p->content[p->cur_pos] = sym;
 	p->keycode[p->cur_pos] = keycode;
 	p->keycode_modifiers[p->cur_pos] = modifier;
-
+	
 	// i18n_content
 	int languages_mask = get_languages_mask();
 	modifier = modifier & (~languages_mask);
@@ -489,19 +549,30 @@ static void buffer_add_symbol(struct _buffer *p, char sym, KeyCode keycode, int
 
 		char *symbol_unchanged = p->keymap->keycode_to_symbol(p->keymap, keycode, i, modifier);
 		if (symbol_unchanged == NULL)
+		{
+			free(symbol);
 			continue;
+		}
 
 		//log_message (ERROR, _("'%c' - '%c' - '%c'"), sym, symbol, symbol_unchanged);
-		p->i18n_content[i].content = (char *) realloc(p->i18n_content[i].content, (strlen(p->i18n_content[i].content) + strlen(symbol) + 1) * sizeof(char));
+		char *tmp = realloc(p->i18n_content[i].content, (strlen(p->i18n_content[i].content) + strlen(symbol) + 1) * sizeof(char));
+		assert(tmp != NULL);
+		p->i18n_content[i].content = tmp;
 		p->i18n_content[i].content = strcat(p->i18n_content[i].content, symbol);
 
-		p->i18n_content[i].content_unchanged = (char *) realloc(p->i18n_content[i].content_unchanged, (strlen(p->i18n_content[i].content_unchanged) + strlen(symbol_unchanged) + 1) * sizeof(char));
+		tmp = realloc(p->i18n_content[i].content_unchanged, (strlen(p->i18n_content[i].content_unchanged) + strlen(symbol_unchanged) + 1) * sizeof(char));
+		assert(tmp != NULL);
+		p->i18n_content[i].content_unchanged = tmp;
 		p->i18n_content[i].content_unchanged = strcat(p->i18n_content[i].content_unchanged, symbol_unchanged);
 
-		p->i18n_content[i].symbol_len = (int *) realloc(p->i18n_content[i].symbol_len, (p->cur_pos + 1) * sizeof(int));
+		tmp = realloc(p->i18n_content[i].symbol_len, (p->cur_pos + 1) * sizeof(int));
+		assert(tmp != NULL);
+		p->i18n_content[i].symbol_len = (int *)tmp;
 		p->i18n_content[i].symbol_len[p->cur_pos] = strlen(symbol);
 
-		p->i18n_content[i].symbol_len_unchanged = (int *) realloc(p->i18n_content[i].symbol_len_unchanged, (p->cur_pos + 1) * sizeof(int));
+		tmp = realloc(p->i18n_content[i].symbol_len_unchanged, (p->cur_pos + 1) * sizeof(int));
+		assert(tmp != NULL);
+		p->i18n_content[i].symbol_len_unchanged = (int *)tmp; 
 		p->i18n_content[i].symbol_len_unchanged[p->cur_pos] = strlen(symbol_unchanged);
 
 		free(symbol);
@@ -554,7 +625,9 @@ static char *buffer_get_utf_string(struct _buffer *p)
 		int nbytes = XLookupString((XKeyEvent *) &event, symbol, 256, NULL, NULL);
 		if (nbytes <= 0)
 			continue;
-
+		if (symbol == NULL)
+			continue;
+		
 		symbol[nbytes] = NULLSYM;
 
 		char *tmp = realloc(utf_string, strlen(utf_string) * sizeof(char) + nbytes + 1);
@@ -566,7 +639,8 @@ static char *buffer_get_utf_string(struct _buffer *p)
 		}
 	}
 
-	free(symbol);
+	if (symbol != NULL)
+		free(symbol);
 	XCloseDisplay(display);
 	
 	return utf_string;
@@ -585,7 +659,7 @@ static char *buffer_get_utf_string_on_kbd_group(struct _buffer *p, int group)
 			state = state & (~keyboard_groups[j]);
 		}
 		char *symbol = p->keymap->keycode_to_symbol(p->keymap, p->keycode[i], group, state);
-		if (symbol)
+		if (symbol != NULL)
 		{
 			char *tmp = realloc(utf_string, strlen(utf_string) * sizeof(char) + strlen(symbol) + 1);
 			if (tmp != NULL)
@@ -624,22 +698,106 @@ static void buffer_unset_offset(struct _buffer *p, int offset)
 	p->cur_pos		+= offset;
 }
 
+int buffer_get_last_word_offset(struct _buffer *p, const char *string, int string_len)
+{
+	// Initial delimeters string concatenation
+	if (strlen(xconfig->delimeters_string) == 0) 
+	{
+		for (int i = 0; i < xconfig->delimeters_count; i++)
+		{
+			char *symbol = p->keymap->keycode_to_symbol(p->keymap, XKeysymToKeycode(p->keymap->display, xconfig->delimeters[i]), -1, 0); 
+			if (strlen(symbol) == 1)
+				strcat(xconfig->delimeters_string, symbol);
+			free(symbol);
+		}
+		//log_message (DEBUG,"'%s'", xconfig->delimeters_string);
+	}
+	// End of initial delimeters string concatenation
+	
+	int len = string_len;
+	while (len != 0 && ((isspace(string[len - 1]) || (strchr(xconfig->delimeters_string, string[len - 1]) != NULL))))
+		len--;
+
+	/*int is_delim;
+	do 
+	{	
+		log_message (DEBUG, "code 0x%x", p->keycode[len - 1]);
+		is_delim = FALSE;
+		for (int i = 0; i < xconfig->delimeters_count; i++)
+		{			
+			if (xconfig->delimeters[i] == p->keycode[len - 1])
+			{
+				is_delim = TRUE;
+			}
+		}
+		if (is_delim)
+			len--;
+	} while ((len != 0) && (is_delim));*/
+	
+	if (len == 0)
+		return string_len;
+
+	while (len != 0 && !isspace(string[len - 1]) && (strchr(xconfig->delimeters_string, string[len - 1]) == NULL))
+		len--;
+
+	/*int is_symbol;
+	do 
+	{
+		is_symbol = TRUE;
+		for (int i = 0; i < xconfig->delimeters_count; i++)
+		{
+			if (xconfig->delimeters[i] == p->keycode[len - 1])
+				is_symbol = FALSE;
+		}
+		if (is_symbol)
+			len--;		
+	} while ((len != 0) && (is_symbol));
+	log_message (DEBUG, "len2 %d", len);*/
+	
+	return len;
+}
+
+char* buffer_get_last_word(struct _buffer *p, char *string)
+{
+	int len = strlen(string);
+
+	int offset = buffer_get_last_word_offset(p, string, len);
+	if (offset == -1)
+		return NULL;
+
+	return string + offset;
+}
+
 static void buffer_uninit(struct _buffer *p)
 {
-	free(p->keycode_modifiers);
-	free(p->keycode);
-	free(p->content);
+	if (p == NULL)
+		return;
+	
+	if (p->keycode_modifiers != NULL)
+		free(p->keycode_modifiers);
+	if (p->keycode != NULL)
+		free(p->keycode);
+	if (p->content != NULL)
+		free(p->content);
 
-	for (int i = 0; i < p->handle->total_languages; i++)
+	if (p->i18n_content != NULL)
 	{
-		free(p->i18n_content[i].content);
-		free(p->i18n_content[i].symbol_len);
-		free(p->i18n_content[i].content_unchanged);
-		free(p->i18n_content[i].symbol_len_unchanged);
-	}
+		for (int i = 0; i < p->handle->total_languages; i++)
+		{
+			if (p->i18n_content[i].content != NULL)
+				free(p->i18n_content[i].content);
+			if (p->i18n_content[i].symbol_len != NULL)
+				free(p->i18n_content[i].symbol_len);
+			if (p->i18n_content[i].content_unchanged != NULL)
+				free(p->i18n_content[i].content_unchanged);
+			if (p->i18n_content[i].symbol_len_unchanged != NULL)
+				free(p->i18n_content[i].symbol_len_unchanged);
+		}
 
-	free(p->i18n_content);
-	free(p);
+		free(p->i18n_content);
+	}
+	if (p != NULL)
+		free(p);
 
 	log_message(DEBUG, _("String is freed"));
 }
@@ -691,6 +849,8 @@ struct _buffer* buffer_init(struct _xneur_handle *handle, struct _keymap *keymap
 	p->get_utf_string_on_kbd_group	= buffer_get_utf_string_on_kbd_group;
 	p->set_offset		= buffer_set_offset;
 	p->unset_offset		= buffer_unset_offset;
+	p->get_last_word	= buffer_get_last_word;
+	p->get_last_word_offset	= buffer_get_last_word_offset;
 	p->uninit		= buffer_uninit;
 
 	return p;
