@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2012 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -22,14 +22,13 @@
 
 #include <X11/XKBlib.h>
 #include <X11/extensions/XInput2.h>
+#include <X11/extensions/XTest.h>
 
-void   set_event_mask(Window window, int event_mask);
-void   set_mask_to_window(Window window, int event_mask);
 char*  get_wm_class_name(Window window);
-void   grab_button(Window window, int is_grab);
-void   grab_keyboard(Window window, int is_grab);
-void   grab_key(Window window, KeyCode kc, int is_grab);
-void   grab_spec_keys(Window window, int is_grab);
-unsigned char *get_win_prop(Window window, Atom atom, long *nitems, Atom *type, int *size); 
+void   grab_button(int is_grab);
+void   grab_all_keys(Window window, int is_grab);
+unsigned char *get_win_prop(Window window, Atom atom, long *nitems, Atom *type, int *size);
+void toggle_lock(int mask, int state);
+void click_key(KeySym keysym);
 
 #endif /* _UTILS_H_ */
