@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2012 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -51,7 +51,7 @@ int is_upper_non_alpha_cyr(char symbol)
 	return FALSE;
 }
 
-int get_last_word_offset(const char *string, int string_len)
+/*int get_last_word_offset(const char *string, int string_len)
 {
 	int len = string_len;
 	while (len != 0 && (isspace(string[len - 1]) || (string[len - 1] == '-')))
@@ -75,7 +75,7 @@ char* get_last_word(char *string)
 		return NULL;
 
 	return string + offset;
-}
+}*/
 
 int trim_word(char *word, int len)
 {
@@ -157,39 +157,59 @@ char* str_replace(const char *source, const char *search, const char *replace)
 
 char* real_sym_to_escaped_sym(const char *source)
 {
-	char *dummy = str_replace(source, "\\", "\\\\");
-	source = strdup(dummy);
-	free(dummy);
+	char *string = strdup(source);
+	if (string == NULL)
+		return NULL;
 
-	dummy = str_replace(source, "\t", "\\t");
-	free((void*)source);
-	source = strdup(dummy);
-	free(dummy);
+	char *dummy = str_replace(string, "\\", "\\\\");
+	if (dummy != NULL)
+	{
+		free(string);
+		string = strdup(dummy);
+		free(dummy);
+	}
 
-	dummy = str_replace(source, "\n", "\\n");
-	free((void*)source);
+	dummy = str_replace(string, "\t", "\\t");
+	if (dummy != NULL)
+	{
+		free(string);
+		string = strdup(dummy);
+		free(dummy);
+	}
 
+	dummy = str_replace(string, "\n", "\\n");
+	free(string);
 	return dummy;
 }
 
 char* escaped_sym_to_real_sym(const char *source)
 {
+	char *string = strdup(source);
+	if (string == NULL)
+		return NULL;
+
 	// Replace escaped-symbols
 	char escape[] = {'\n', NULLSYM};
-	char *dummy = str_replace(source, "\\n", escape);
-	source = strdup(dummy);
-	free(dummy);
+	char *dummy = str_replace(string, "\\n", escape);
+	if (dummy != NULL)
+	{
+		free(string);
+		string = strdup(dummy);
+		free(dummy);
+	}
 
 	escape[0] = '\t';
-	dummy = str_replace(source, "\\t", escape);
-	free((void*)source);
-	source = strdup(dummy);
-	free(dummy);
+	dummy = str_replace(string, "\\t", escape);
+	if (dummy != NULL)
+	{
+		free(string);
+		string = strdup(dummy);
+		free(dummy);
+	}
 
 	escape[0] = '\\';
-	dummy = str_replace(source, "\\\\", escape);
-	free((void*)source);
-
+	dummy = str_replace(string, "\\\\", escape);
+	free(string);
 	return dummy;
 }
 
