@@ -13,7 +13,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  Copyright (C) 2006-2013 XNeur Team
+ *  Copyright (C) 2006-2016 XNeur Team
  *
  */
 
@@ -100,7 +100,7 @@ static void xneur_reload(int status);
 
 static void xneur_init(void)
 {
-	bind_manual_actions();
+	bind_actions();
 	bind_user_actions();
 }
 
@@ -212,6 +212,13 @@ static void xneur_load_config(void)
 	log_message(LOG, _("Tracking input mode set to %s"), _(xconfig->get_bool_name(xconfig->tracking_input)));
 	log_message(LOG, _("Tracking mouse mode set to %s"), _(xconfig->get_bool_name(xconfig->tracking_mouse)));
 	log_message(LOG, _("Delay before sendind events to application set to (in milliseconds) %d"), xconfig->send_delay);
+
+	// Delimeters
+	log_message(LOG, _("Word Delimeters:"));
+	for (int i = 0; i < xconfig->delimeters_count; i++)
+	{
+		log_message(LOG, "    %s (0x%x)", XKeysymToString(xconfig->delimeters[i]), xconfig->delimeters[i]);
+	}
 }
 
 static void xneur_set_lock(void)
@@ -238,6 +245,8 @@ static void xneur_cleanup(void)
 	sound_uninit();
 	log_message(DEBUG, _("Current sound data is freed"));
 
+	unbind_actions();
+	log_message(DEBUG, _("Current keybinds is freed"));
 	unbind_user_actions();
 	log_message(DEBUG, _("Current user keybinds is freed"));
 	
@@ -289,9 +298,10 @@ static void xneur_reload(int status)
 	show_notify(NOTIFY_XNEUR_RELOAD, NULL);
 	
 	sound_uninit();
+	unbind_actions();
 	unbind_user_actions();
 	program->plugin->xneur_reload(program->plugin);
-	
+
 	if (xconfig != NULL)
 		xconfig->uninit(xconfig);
 
@@ -415,7 +425,7 @@ static void xneur_trap(int sig, sg_handler handler)
 
 	if (sigaction(sig, &sa, NULL) == -1)
 	{
-		log_message(ERROR, _("Can't trap signal"));
+		log_message(ERROR, _("Can't trap signal %d"), sig);
 		exit(EXIT_FAILURE);
 	}
 }
@@ -432,6 +442,7 @@ int main(int argc, char *argv[])
 	xneur_trap(SIGINT, xneur_terminate);
 	xneur_trap(SIGHUP, xneur_reload);
 	xneur_trap(SIGCHLD, xneur_zombie);
+	xneur_trap(SIGTSTP, xneur_terminate);
 	
 	xneur_get_options(argc, argv);
 	
@@ -472,7 +483,7 @@ int main(int argc, char *argv[])
 	show_notify(NOTIFY_XNEUR_START, NULL);
 
 	program->plugin->xneur_start(program->plugin);
-			
+
 	if (xneur_generate_proto)
 		generate_protos();
 	else	
